library(Kendall)

pval_diff <- read.csv("ATestData.csv",header=T,check.names=FALSE) ## Differentially expressed dataset
pval_diff[1:5,1:5]

dim(pval_diff)
n1 <- 500        ## control group
n2 <- 500        ## case group

TFsexpress <- pval_diff[,which(colnames(pval_diff)%in%paste("TF",1:10,sep=""))]  ## TFs expression dataset
dim(TFsexpress)
TFsexpress[1:5,1:5]

full_data <- pval_diff[,-1]  ## full dataset

exp_new <- full_data[1:500,]   ## case group
ctl_new <- full_data[501:1000,]  ## control group


comm_ctl_exp=intersect(colnames(exp_new),colnames(ctl_new))
ctl_new1=ctl_new[,which(colnames(ctl_new)%in%comm_ctl_exp)]
exp_new1=exp_new[,which(colnames(exp_new)%in%comm_ctl_exp)]
dim(exp_new1)
dim(ctl_new1)
TFs <- colnames(ctl_new1)[colnames(ctl_new1)%in%colnames(TFsexpress)]  ## TFs

use_TFs <- TFs
only_genes <- colnames(ctl_new1)[!colnames(ctl_new1)%in%TFs]  ## genes not TFs

## calculating the differential connectivity
s <- NULL
for(l in 1:length(use_TFs)){
sub <- c(use_TFs[l],only_genes)  
pop_exp_new <- exp_new1[,colnames(exp_new1)%in%sub]
pop_ctl_new <- ctl_new1[,colnames(ctl_new1)%in%sub]

s1=cor(pop_exp_new[,use_TFs[l]],pop_exp_new,method="pearson")
s2=cor(pop_ctl_new[,use_TFs[l]],pop_ctl_new,method="pearson")
s[l] <- sum(abs(s1-s2))/(length(only_genes))
}
DEgenes_pop <- data.frame(use_TFs,s)
TfTestStat_pop=DEgenes_pop[with(DEgenes_pop, order(-DEgenes_pop$s)),]
                           ## Differential connectivity scores for TFs
#write.csv(TfTestStat_pop,"DEindividualtestpval1.csv")


#########################################################################
## Step2b)


PulledTFexpressTrans=TFsexpress[,colnames(TFsexpress)%in%TFs]  ## Expression set for TFs only
PulledTFexpressTrans[1:5,1:5]
dim(PulledTFexpressTrans)

temp <- PulledTFexpressTrans[, !duplicated(colnames(PulledTFexpressTrans))]

cormatTF=cor(temp,method="pearson")  ## Correlation among TFs

#####################################################################
## Step 3 & 4

#TfTestStat_pop=read.csv("DEindividualtestpval1.csv",header=T,check.names=FALSE)[,-1]

## Calculating the Kendall's statistic
stat_tau=NULL
for(t in 1:length(TfTestStat_pop[,1])){
TranInterested=as.character(TfTestStat_pop[t,1])

orderCorrTF=sort(abs(cormatTF[TranInterested,]),decreasing=TRUE)

d1=data.frame(gene=TfTestStat_pop[,1],TfTestStat_pop)
d2=data.frame(orderCorrTF)
d22=data.frame(d2,gene=rownames(d2))
data=merge(d22, d1, by="gene")

test_tau=Kendall(data[,2], data[,4])
stat_tau[t]=test_tau$tau
}
allTFsStat_pop=data.frame(TransFacs=TfTestStat_pop[,1],stat_tau)  ## Kendall's statistic value for all TFs
allTFsStat_pop1 <- allTFsStat_pop[with(allTFsStat_pop,order(-allTFsStat_pop[,2])),]
                                      ## Ordered list of TFs based on their test statistic values.



write.csv(allTFsStat_pop1,"allTFsStat_pop1.csv")









############################  P-Value calculation #######################################################

###########################################################################################################
##############################################################################################################
## Sampling (p-value calculation)

###############################################################################
full_data <- rbind(exp_new1,ctl_new1)
all_genes <- unique(colnames(exp_new1))
only_genes <- all_genes[!all_genes%in%TFs]

p_tau <- NULL
for(k in 1:500){ ## change   (No. of iterations)
n <- 1000


x <- sample(1:1000,size=n,replace=TRUE)
sam_exp=full_data[x[1:500],]
sam_ctl=full_data[x[501:1000],]                ## Bootstrap samples

## Bootstrap based differential connectivity
s <- NULL
for(l in 1:length(use_TFs)){  
sub <- c(use_TFs[l],only_genes) 
sam_exp_new <- sam_exp[,colnames(sam_exp)%in%sub]
sam_ctl_new <- sam_ctl[,colnames(sam_ctl)%in%sub]

s1=cor(sam_exp_new[,use_TFs[l]],sam_exp_new,method="pearson")
s2=cor(sam_ctl_new[,use_TFs[l]],sam_ctl_new,method="pearson")
s[l] <- sum(abs(s1-s2))/length(only_genes)         
}
DEgenes_sam <- data.frame(use_TFs,s) 
TfTestStat=DEgenes_sam[with(DEgenes_sam, order(-DEgenes_sam$s)),]


data_req=rbind(sam_ctl,sam_exp)
TFexpress=data_req[,which(colnames(data_req)%in%TFs)]

cormatTF=cor(TFexpress,method="pearson")

## Bootstrap based Kendall's test statistic values
stat=NULL
for(t in 1:length(TFs)){ 
TranInterested=as.character(TfTestStat[t,1])

orderCorrTF=sort(abs(cormatTF[TranInterested,]),decreasing=TRUE)

d1=data.frame(gene=TfTestStat[,1],TfTestStat)
d2=data.frame(orderCorrTF)
d22=data.frame(d2,gene=rownames(d2))
data=merge(d22, d1, by="gene")

test_tau=Kendall(data[,2], data[,4])
stat[t]=test_tau$tau
}
allTFsStat=data.frame(TransFacs=TfTestStat[,1],stat) 
Stat_obs_tau=max(allTFsStat[,2])
p_tau[k] <- Stat_obs_tau

status <- p_tau  ## Bootstrap based test statistic values
}
#write.csv(status,"statusvec1.csv")







