library(MASS)
library(dna)
library(Kendall)

pvalue_boot_tau <- NULL

for(z in 1:500){


### case group

mu_u <- rep(0,10)           
Sigma=diag(10)
u <- mvrnorm(500,mu_u,Sigma)

colnames(u) <- paste("u",1:10,sep="")   ##  Vector V

###############################################################
## TFs 1 to 10

x1 <- rnorm(500,mean=50,sd=1)                      ## TF1
rho <- c(0,0.95,0.8,0.7,0.6,0.5,0.4,0.3,0.2,0.1)   ## Rho vector


s <- x1
for(i in 2:10){
xi <- (rho[i]*x1+u[,i])/sqrt(1+(rho[i])^2)
s <- cbind(s,xi)
}
colnames(s) <- paste("TF",1:10,sep="")            ## TF1 to TF10
round(cor(s),4) 

####################################################################
## Genes 1 to 30 
delta <- 1                                             ## Delta
r1 <- c(0.45,0.4,0.35,0.3,0.25,0.2,0.15,0.1,0.05)      ## r1s
r2 <- (1-delta)*r1                                     ## r2s
gamma1 <- 0.5+(delta^2)*r1[1]                          ## Gamma1

genes1to30 <- NULL
for(i in 1:30){
e1 <- rnorm(500)
g1 <- x1*gamma1+e1
genes1to30 <- cbind(genes1to30,g1)
}
colnames(genes1to30) <- paste("G",1:30,sep="")         ## Genes1 to 30

m <- c(10,10,10,10,10,10,5,5,5)                        ## m vector
r1 <- c(0.45,0.4,0.35,0.3,0.25,0.2,0.15,0.1,0.05)       
oth_genes <- NULL
for(i in 2:10){
for(j in 1:m[i-1]){
e <- rnorm(500)
gj <- u[,i]*r1[i-1]+e
oth_genes <- cbind(oth_genes,gj)
}
}
colnames(oth_genes) <- paste("G",31:105,sep="")        ## Remaining genes


pop_exp <- cbind(s,genes1to30,oth_genes)               ## case group : TFs, Genes

############################################################################
############################################################################

### control group
mu_u <- rep(0,10)
Sigma=diag(10)
u <- mvrnorm(500,mu_u,Sigma)
colnames(u) <- paste("u",1:10,sep="")                  ## Vector V

###############################################################
## TFs 1 to 10

x1 <- rnorm(500,mean=5,sd=1)                            ## TF1
rho <- c(0,0.95,0.8,0.7,0.6,0.5,0.4,0.3,0.2,0.1)       ## same Rho


s <- x1
for(i in 2:10){
xi <- (rho[i]*x1+u[,i])/sqrt(1+(rho[i])^2)
s <- cbind(s,xi)
}
colnames(s) <- paste("TF",1:10,sep="")                 ## TF1 to TF10
round(cor(s),4) 

####################################################################
## Genes 1 to 30

gamma2 <- 0.5
genes1to30 <- NULL
for(i in 1:30){
e1 <- rnorm(500)
g1 <- x1*gamma2+e1
genes1to30 <- cbind(genes1to30,g1)
}
colnames(genes1to30) <- paste("G",1:30,sep="")           ## Genes 1 to 30

m <- c(10,10,10,10,10,10,5,5,5)
r2 <- (1-delta)*r1
oth_genes <- NULL
for(i in 2:10){
for(j in 1:m[i-1]){
e <- rnorm(500)
gj <- u[,i]*r2[i-1]+e
oth_genes <- cbind(oth_genes,gj)
}
}
colnames(oth_genes) <- paste("G",31:105,sep="")          ## Remaining genes

pop_ctl <- cbind(s,genes1to30,oth_genes)                 ## control group: TFs and genes

#############################################################################
#############################################################################
TFs <- paste("TF",1:10,sep="")
genes <- paste("G",1:105,sep="")

## Measuring differential connectivity scores for all TFs
s <- NULL
for(l in 1:length(TFs)){
sub <- c(TFs[l],genes) 
pop_exp_new <- pop_exp[,colnames(pop_exp)%in%sub]
pop_ctl_new <- pop_ctl[,colnames(pop_ctl)%in%sub]

s1=cornet(pop_exp_new,rescale.scores=FALSE)
s2=cornet(pop_ctl_new,rescale.scores=FALSE)
s[l] <- sum(abs(s1[,TFs[l]]-s2[,TFs[l]]))/(length(genes))
}
DEgenes_pop <- data.frame(TFs,s)
DEgenes_pop
TfTestStat_pop=DEgenes_pop[with(DEgenes_pop, order(-DEgenes_pop$s)),]
TfTestStat_pop


## Pearson's correlation between each pair of TFs
data_req_pop=rbind(pop_exp,pop_ctl)
TFexpress_pop=data_req_pop[,which(colnames(data_req_pop)%in%TFs)]
cormatTF_pop=cor(TFexpress_pop,method="pearson")


## Kendall's statistic for each TF
stat_tau=NULL
for(t in 1:length(TFs)){
TranInterested=as.character(TfTestStat_pop[t,1])
orderCorrTF=sort(abs(cormatTF_pop[TranInterested,]),decreasing=TRUE)

d1=data.frame(gene=TfTestStat_pop[,1],TfTestStat_pop)
d2=data.frame(orderCorrTF)
d22=data.frame(d2,gene=rownames(d2))
data=merge(d22, d1, by="gene")

test_tau=Kendall(data[,2], data[,4])
stat_tau[t]=test_tau$tau
}
allTFsStat_pop=data.frame(TfTestStat_pop[,1],stat_tau)
Stat_pop_tau=max(allTFsStat_pop[,2])                           ## Final test statistic K

########################################################################################

###############################################################################
##############   Sampling   ###################################################

full_pop <- rbind(pop_exp,pop_ctl)
p_tau <- NULL
for(k in 1:500){ 


## Bootstrap samples
x <- seq(1:1000)   
sam <- sample(x,1000,replace=TRUE) 
sam_exp=full_pop[sam[1:500],]      
sam_ctl=full_pop[sam[501:1000],]   


## Differential connectivity scores for all TFs for a bootstrap sample
s <- NULL
for(l in 1:length(TFs)){
sub <- c(TFs[l],genes) 
sam_exp_new <- sam_exp[,colnames(sam_exp)%in%sub]
sam_ctl_new <- sam_ctl[,colnames(sam_ctl)%in%sub]

s1=cornet(sam_exp_new,rescale.scores=FALSE)
s2=cornet(sam_ctl_new,rescale.scores=FALSE)
s[l] <- sum(abs(s1[,TFs[l]]-s2[,TFs[l]]))/(length(genes))
}
DEgenes_sam <- data.frame(TFs,s)
DEgenes_sam

TfTestStat=DEgenes_sam[with(DEgenes_sam, order(-DEgenes_sam$s)),]
TfTestStat


## Pearson's correlation among the TFs for a bootstrap sample
data_req=rbind(sam_exp,sam_ctl)
TFexpress=data_req[,which(colnames(data_req)%in%TFs)]
cormatTF=cor(TFexpress,method="pearson")


## Kendall's test statistic value for each TF for a bootstrap sample
stat_tau=NULL
for(t in 1:length(TFs)){
TranInterested=as.character(TfTestStat[t,1])
orderCorrTF=sort(abs(cormatTF[TranInterested,]),decreasing=TRUE)

d1=data.frame(gene=TfTestStat[,1],TfTestStat)
d2=data.frame(orderCorrTF)
d22=data.frame(d2,gene=rownames(d2))
data=merge(d22, d1, by="gene")

test_tau=Kendall(data[,2], data[,4])
stat_tau[t]=test_tau$tau
}
allTFsStat=data.frame(TfTestStat[,1],stat_tau)
Stat_obs_tau=max(allTFsStat[,2])               ## Bootstrap based test statistic value
p_tau[k] <- Stat_obs_tau
}
pvalue_boot_tau[z]=mean(p_tau > Stat_pop_tau)  ## p-value calculation based on 500 bootstrap samples
print(z)
}
datafin <- data.frame(seq(1:500),pvalue_boot_tau)
sum(datafin[,2]<0.05)




